--------------------------------------------------------------------------------
-- Company:    Digilent inc
-- Engineer:   Kovacs Laszlo - Attila
--
-- Create Date:    18:49:34 03/25/07
-- Design Name:    
-- Module Name:    EPP - Behavioral
-- Project Name:   
-- Target Device:  X-Board, PModRS232
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity EPP is
    Port ( 
      Clk      : in  std_logic;

      EppAstb  : in  std_logic;
      EppDstb  : in  std_logic;
      EppWrite : in  std_logic;
      EppWait  : out std_logic;
      EppRst   : in  std_logic;
      EppDB    : inout std_logic_vector(7 downto 0);

      Address  : out std_logic_vector(7 downto 0);
      DataOut  : out std_logic_vector(7 downto 0);
      Read     : out std_logic;
      Write    : out std_logic;

      DataIn   : in std_logic_vector(7 downto 0);
      DataRdy  : in std_logic);

end EPP;

architecture Behavioral of EPP is

signal addr  : std_logic_vector(7 downto 0) := x"00";
signal prevAstb, prevDstb : std_logic := '0';
signal astb, dstb : std_logic := '0';

begin
   
   -- drive EPP Data Bus when Read is performed
   EppDB <= 
      DataIn when EppWrite = '1' and 
         (EppAstb = '0' or EppDstb = '0') else 
      (others => 'Z');
   
   -- EPP address
   Address <= addr;
   
   -- the data from the EPPDB is
   -- provided for the other modules in the project
   DataOut <= EppDB;
   
   -- detect Address and Data Strobes form the current and 
   -- the value stored in register
   astb <= '1' when prevAstb = '1' and EppAstb = '0' else '0';
   dstb <= '1' when prevDstb = '1' and EppDstb = '0' else '0';
      
   
   process(Clk, EppRst, EppAstb, EppDstb)
   begin
      if EppRst = '0' then
         prevAstb <= EppAstb;
         prevDstb <= EppDstb;
         EppWait <= '0';
      elsif rising_edge(Clk) then

         -- store these values for strobe detection
         prevAstb <= EppAstb;
         prevDstb <= EppDstb;
         
         -- latch the address from the data bus when Address strobe detected
         if astb = '1' then 
            addr <= EppDB;
         end if;
         
         -- signal Write for the other modules in the project
         if dstb = '1' and EppWrite = '0' then
            Write <= '1';
         else
            Write <= '0';
         end if;
         
         -- signal Read for the other modules in the project
         if dstb = '1' and EppWrite = '1' then
            Read <= '1';
         else
            Read <= '0';
         end if;
         
         -- bring low Wait when both control signals are high
         -- activate Wait when:
         --  address strobe detected, handled in this module
         --  the addressed module is ready for the data transfer
         if EppAstb = '1' and EppDstb = '1' then
            EppWait <= '0';
         elsif astb = '1' or DataRdy = '1' then 
            EppWait <= '1';
         end if;
         
      end if;
   end process;

   

end Behavioral;
